@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2014 Pavel 'Blane' Tuchin
+# Copyright (c) 2021 Pavel 'Blane' Tuchin
 # This file is part of pynetdicom2, released under a modified MIT license.
 #    See the file license.txt included with this distribution.
 
+# pylint: disable=line-too-long
 """
 This module provides some useful status constants. You should be aware, that some status codes depends on a command
 or a service you are responding to. The following tables provides list of some well known status codes grouped by
@@ -303,27 +304,28 @@ New status codes can be created using ``statuses.Status`` class::
 
     SUCCESS = Status(0x0000)
 
-``statuses.Status`` class also takes an optional argument ``command`` to create status depending on command.
+``statuses.Status`` class also takes an optional argument ``command`` to create status depending on the command.
 This class is a convenience wrapper around integer, that provides information on status type (Failure, Success, Warning
 or Pending) and status description. If status code is unknown to the library it assumes it's a failure with unspecified
 description.
 
 You can add new statuses by using ``statuses.add_status`` function. By using it you will be adding new known status code
-to a global library dictionary of status codes.
+to the global library dictionary of status codes.
 
 """
+# pylint: enable=line-too-long
 
 __author__ = 'Blane'
 from collections import namedtuple
 
 from . import dimsemessages as dimse
 
-s = namedtuple('status', ['code_type', 'description'])
+s = namedtuple('status', ['code_type', 'description'])  # pylint: disable=invalid-name
 
 
-_general_status_dict = {}
+_general_status_dict = {}  # pylint: disable=invalid-name
 
-_status_dict = {}
+_status_dict = {}  # pylint: disable=invalid-name
 
 
 UNKNOWN = s('Failure', 'Unknown Status')
@@ -352,36 +354,43 @@ def add_status(code, code_type, description, end=None, command=None):
             _status_dict[(command.command_field, _code)] = status
 
 
-class Status(int):
+class Status(object):
     """Class represents message status.
 
     This is a helper class that provides convenience methods for printing status codes.
     """
 
-    def __new__(cls, value, command=None):
-        """Initializes new Status .
+    def __init__(self, value, command=None):
+        """Initializes new Status.
 
         :param value status code
         :param command: command for which status is created. Some codes depends on the
                         type of service.
         """
-        obj = super(Status, cls).__new__(cls, value)
+        self._value = value
         status = None
         if command:
             status = _status_dict.get((command.command_field, value))
         if not status:
             status = _general_status_dict.get(value, UNKNOWN)
-        obj.status_type = status.code_type
-        obj.description = status.description
-        obj.is_success = obj.status_type == 'Success'
-        obj.is_pending = obj.status_type == 'Pending'
-        obj.is_failure = obj.status_type == 'Failure'
-        obj.is_warning = obj.status_type == 'Warning'
-        obj.is_cancel = obj.status_type == 'Cancel'
-        return obj
+        self.status_type = status.code_type
+        self.description = status.description
+
+        self.is_success = self.status_type == 'Success'
+        self.is_pending = self.status_type == 'Pending'
+        self.is_failure = self.status_type == 'Failure'
+        self.is_warning = self.status_type == 'Warning'
+        self.is_cancel = self.status_type == 'Cancel'
+
+    def __int__(self):
+        return int(self._value)
 
     def __str__(self):
-        return '(0x{value:0X}) {self.status_type}: {self.description}'.format(self=self, value=int(self))
+        # pylint: disable=missing-format-attribute
+        return '(0x{value:0X}) {self.status_type}: {self.description}'.format(
+            self=self, value=int(self)
+        )
+        # pylint: enable=missing-format-attribute
 
     def __repr__(self):
         """Returns status string representation
@@ -391,6 +400,7 @@ class Status(int):
         return 'Status(0x{self:0X})'.format(self=int(self))
 
 
+# pylint: disable=line-too-long
 KNOWN_STATUSES = [
     (0x0000, 'Success', '', None),
     (0x0105, 'Failure', 'No Such Attribute', None),
@@ -461,9 +471,11 @@ KNOWN_STATUSES = [
     (0xB000, 'Warning', 'Sub-operations Complete - One or more Failures or Warnings', dimse.CMoveRSPMessage),
     (0xFF00, 'Pending', 'Sub-operations are continuing', dimse.CMoveRSPMessage)
 ]
+# pylint: enable=line-too-long
 
 
 def register_statuses():
+    """Registers known statuses in the global library dictionary"""
     for status in KNOWN_STATUSES:
         code, code_type, desc, command = status
         if isinstance(code, tuple):
@@ -487,12 +499,18 @@ C_STORE_OUT_OF_RESOURCES = Status(0xA700, dimse.CStoreRSPMessage)
 #: (0xB006) Elements Discarded (C-STORE)
 C_STORE_ELEMENTS_DISCARDED = Status(0xB006, dimse.CStoreRSPMessage)
 
-#: (0xFF00) Matches are continuing - Current Match is supplied and any Optional Keys were supported in the same
-#: manner as Required Keys. (C-FIND)
 C_FIND_PENDING = Status(0xFF00, dimse.CFindRSPMessage)
-#: (0xFF01) Matches are continuing - Warning that one or more Optional Keys were not
-#: supported for existence and/or matching for this Identifier. (C-FIND)
+"""
+(0xFF00) Matches are continuing - Current Match is supplied and any Optional
+Keys were supported in the same manner as Required Keys. (C-FIND)
+"""
+
 C_FIND_PENDING_WARNING = Status(0xFF01, dimse.CFindRSPMessage)
+"""
+(0xFF01) Matches are continuing - Warning that one or more Optional Keys were not
+supported for existence and/or matching for this Identifier. (C-FIND)
+"""
+
 #: (0xC000) Failed: Unable to process (C-FIND)
 C_FIND_UNABLE_TO_PROCESS = Status(0xC000, dimse.CFindRSPMessage)
 #: (0xA700) Refused: Out of Resources (C-FIND)
